"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const archiver_1 = __importDefault(require("archiver"));
const fs_1 = __importDefault(require("fs"));
function zipFile(src, dest) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            // create a file to stream archive data to.
            var output = fs_1.default.createWriteStream(dest);
            var archive = archiver_1.default("zip", {
                zlib: { level: 9 },
            });
            output.on("close", () => {
                resolve();
            });
            archive.on("error", reject);
            archive.file(src, {
                name: path_1.default.basename(src),
            });
            archive.pipe(output);
            archive.finalize();
        });
    });
}
exports.zipFile = zipFile;
function zipDir(src, dest, ignore) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            // create a file to stream archive data to.
            var output = fs_1.default.createWriteStream(dest);
            var archive = archiver_1.default("zip", {
                zlib: { level: 9 },
            });
            output.on("close", () => {
                resolve();
            });
            archive.on("error", reject);
            archive.glob("**/*", {
                cwd: src,
                ignore: ignore || [],
                dot: true,
            }, {});
            archive.pipe(output);
            archive.finalize();
        });
    });
}
exports.zipDir = zipDir;
class FunctionBuilder extends Builder {
    constructor(options) {
        super(Object.assign({ type: "function" }, options));
    }
    build(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const functions = yield Promise.all(options.map((option) => __awaiter(this, void 0, void 0, function* () {
                const localZipPath = path_1.default.join(this.distDir, option.zipFileName);
                if (!fs_1.default.existsSync(this.distDir)) {
                    mkdirSync(this.distDir);
                }
                if (!fs_1.default.existsSync(option.localPath)) {
                    throw new Error(`函数目录或者文件 ${path_1.default.basename(option.localPath)} 不存在`);
                }
                const fileStats = fs_1.default.statSync(option.localPath);
                if (fileStats.isFile()) {
                    this.logger.debug("option.localPath", option.localPath, localZipPath);
                    yield this.zipFile(option.localPath, localZipPath);
                }
                else if (fileStats.isDirectory()) {
                    this.logger.debug("option.localPath", option.localPath, localZipPath);
                    yield this.zipDir(option.localPath, localZipPath, option.ignore);
                }
                return {
                    name: option.name,
                    options: {},
                    source: localZipPath,
                    entry: option.zipFileName,
                };
            })));
            return {
                functions,
            };
        });
    }
    zipFile(src, dest) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                // create a file to stream archive data to.
                var output = fs_1.default.createWriteStream(dest);
                var archive = archiver_1.default("zip", {
                    zlib: { level: 9 },
                });
                output.on("close", () => {
                    resolve();
                });
                archive.on("error", reject);
                archive.file(src, {
                    name: path_1.default.basename(src),
                });
                archive.pipe(output);
                archive.finalize();
            });
        });
    }
    zipDir(src, dest, ignore) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                // create a file to stream archive data to.
                var output = fs_1.default.createWriteStream(dest);
                var archive = archiver_1.default("zip", {
                    zlib: { level: 9 },
                });
                output.on("close", () => {
                    resolve();
                });
                archive.on("error", reject);
                archive.glob("**/*", {
                    cwd: src,
                    ignore: ignore || [],
                    dot: true,
                }, {});
                archive.pipe(output);
                archive.finalize();
            });
        });
    }
}
exports.FunctionBuilder = FunctionBuilder;
